@@ -1,18 +1,17 @@
 # Example demonstrating XOR with momentum backprop learning
 
 use strict;
-use AI::NNFlex;
+use AI::NNFlex::momentum;
 use AI::NNFlex::Dataset;
 
 # Create the network 
 
-my $network = AI::NNFlex->new(randomconnections=>0,
-				networktype=>'feedforward',
+my $network = AI::NNFlex::momentum->new(randomconnections=>0,
 				randomweights=>1,
-				learningalgorithm=>'momentum',
 				learningrate=>.1,
 				debug=>[],bias=>1,
-				momentum=>0.6);
+				momentum=>0.6,
+				round=>1);
 
 
 
@@ -21,8 +20,7 @@ $network->add_layer(	nodes=>2,
 			decay=>0.0,
 			randomactivation=>0,
 			threshold=>0.0,
-			activationfunction=>"tanh",
-			randomweights=>1);
+			activationfunction=>"tanh");
 
 
 $network->add_layer(	nodes=>2,
@@ -30,16 +28,14 @@ $network->add_layer(	nodes=>2,
 			decay=>0.0,
 			randomactivation=>0,
 			threshold=>0.0,
-			activationfunction=>"tanh",
-			randomweights=>1);
+			activationfunction=>"tanh");
 
 $network->add_layer(	nodes=>1,
 			persistentactivation=>0,
 			decay=>0.0,
 			randomactivation=>0,
 			threshold=>0.0,
-			activationfunction=>"linear",
-			randomweights=>1);
+			activationfunction=>"linear");
 
 
 $network->init();
@@ -50,6 +46,8 @@ my $dataset = AI::NNFlex::Dataset->new([
 			[1,0],[1],
 			[1,1],[0]]);
 
+
+
 my $counter=0;
 my $err = 10;
 while ($err >.001)
